#ifndef IMSS_H
#define IMSS_H

extern i_img *
imss_win32(unsigned hwnd, int include_decor, int left, int top, int right, int bottom);

extern i_img *
imss_x11(unsigned long display, int window_id, int left, int top, int right, int bottom);

extern unsigned long
imss_x11_open(char const *display_name);
extern void
imss_x11_close(unsigned long display);

#endif
